/*  Simple views of a box, for initial testing of graphics stuff.	*/
/*  File:  box.cs							*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "Xu.hcs"
#include "Xuf.hcs"
#include "view.h"
#include "persp.hs"

/* The screen */

#define Ht 512
#define Wd 512

shape [Wd][Ht]Screen;		/* Used for everything	*/

int:Screen I, J;	/* Coordinate pvars (axis 0, 1)	*/

/* The object representation - a set of points on surfaces */

float:Screen X[3],	/* Point position	*/
	   Nrm[3];	/* Outward normal	*/

int:Screen Iclr,	/* Intrinsic color, > 0 in active entry	*/
	   Vclr;	/* Color as viewed			*/

/* The image */

int:Screen img;		/* Pixel color codes	*/

/* ==================================================================== */

/* Make rectangle in the coordinate rectangle [ia:ib] x [ja:jb] with	*/
/* normal in coordinate direction c at distance hc and physical		*/
/* dimensions 2*hi x 2*hj , color clr.  `i' corresponds to physical	*/
/* xi, `j' corresponds to physical axis xj.  Call with Screen shape	*/
/* selected, at least region of interest active.			*/

void mkrect(int ia, int ib, int ja, int jb, int ix, int jx,
	    int c,  int clr, float hc, float hi, float hj  )
 { int i, u, ua[2], ub[2], ux[2];  float s, uh[2];
   printf("Rect:  points: [%3d:%3d]x[%3d:%3d]  axis: %1d  clr: %1d\n",
		            ia, ib,   ja, jb,         c,        clr   );
   printf(" disp: %7.2f  half-dimensions:  %7.2f  %7.2f\n",
		    hc,			     hi,    hj     );
   s = (hc > 0.0f) ? 1.0f : -1.0f;	/* Direction of normal	*/
   ua[0] = ia;  ua[1] = ja;  ub[0] = ib;  ub[1] = jb;
   ux[0] = ix;  ux[1] = jx;  uh[0] = hi;  uh[1] = hj;
   where ( (ia <= I) & (I <= ib) & (ja <= J) & (J <= jb) )
   { Nrm[c] = s;  X[c] = hc;
     for ( i = 0;  i < 2;  i++ )
      { Nrm[ux[i]] = 0.0f;
        X[ux[i]] = (uh[i]*(2*(i?J:I) - ua[i] - ub[i]))/(ub[i] - ua[i]);
      }
     Iclr = clr;		/* Intrinsic color */
   }
 }

/* ==================================================================== */

/* Make box centered on origin of dimensions dl, dw, dh, with faces	*/
/* represented by point arrays [pl x pw], [pl x ph], [pw x ph] .	*/
/* Requires  3*ph + 2*pw <= Ht,   pl <= Wd,  and  2*pw <= Wd .		*/
/* Uses colors 1, 2, 3, opposite faces have same color.			*/

void mkbox(int pl, int pw, int ph, float dl, float dw, float dh)
 { float hl, hw, hh;  int v;
   hl = dl/2.0f;  hw = dw/2.0f;  hh = dh/2.0f;
   v = 0;
   mkrect( 0,   pw-1, v,   ph-1, 0, 2, 1, 1,  hl, hw, hh);
   mkrect(pw, 2*pw-1, v,   ph-1, 0, 2, 1, 1, -hl, hw, hh);
   v = ph;
   mkrect( 0,   pl-1, v, v+pw-1, 1, 0, 2, 2,  hh, hl, hw);
   v += pw;
   mkrect( 0,   pl-1, v, v+pw-1, 1, 0, 2, 2, -hh, hl, hw);
   v += pw;
   mkrect( 0,   pl-1, v, v+ph-1, 1, 2, 0, 3,  hw, hl, hh);
   v += ph;
   mkrect( 0,   pl-1, v, v+ph-1, 1, 2, 0, 3, -hw, hl, hh);
 }


/* ==================================================================== */

/*  Initialize screen and display system.  Defines colors and leaves	*/
/*  screen with background color (relative 0).				*/

void Initscreen()
 { int newclr, newbas;
   Xinit(Wd, Ht);
   newclr = 1;  alloclrs(&newclr, &newbas);	/* Assumes success */
   storecolor(0, 60000, 60000, 60500);		/* Background */
   with (Screen) everywhere
    { I = pcoord(0);  J = pcoord(1);
      img = 0;  Xdraw(img);
    }
   loadcolors("pal8.colors", &newclr);		/* Assumes success */   
 }


/* ==================================================================== */

/*  View for this test							*/

View Vw;
float cen[3] = {10.0f, 10.0f, 10.0f}, 	/* Position of screen center	*/
         vdn = 2.54f*18.0f;		/* Viewing distance		*/

/*  Initialize view							*/

void Initvw()
 { vraster(&Vw, Wd, Ht, 2.54f/90.0f);	/* 90 pixels/inch */
   Standvw(&Vw, cen, vdn);
   disp_view(&Vw);
 }


/* ==================================================================== */

/*  Initialize the test box 						*/

int Boxd[3] = {10, 5, 4};	/* Box dimensions	*/

void Initbox()
 { int pl, pw, ph, q;
   with (Screen) everywhere
   { Iclr = 0;		/* Will permit identification of active entries	*/
     pl = Wd;  q = Ht/(2*Boxd[1] + 3*Boxd[2]);
     pw = q*Boxd[1];  ph = q*Boxd[2];
     mkbox(pl, pw, ph, (float)Boxd[0], (float)Boxd[1], (float)Boxd[2]);
   }
 }


/* ==================================================================== */

/*  Main program to set and display test image.				*/

void main()
 { int:Screen ph, pv;	/* Plot coordinates	*/
   char m[128];		/* For keyboard signal	*/
   Initscreen();
   Initvw();
   Initbox();
   /*  Project box and draw it. */
   with (Screen) everywhere
    { Xdraw(Iclr);
      printf("Type p <ret> to proceed > ");
      scanf("%1s", &m);
      where ( Iclr > 0 )
       { /* Diagnostics */
	 printf("%d cells are active.\n", (+= (int:current) 1));
	 printf("Status of cell [0][0]:\n");
         printf("            X                       Nrm              Iclr\n");
         printf("[%7.2f %7.2f %7.2f]  [%7.2f %7.2f %7.2f]  %4d\n",
		  [0][0]X[0], [0][0]X[1], [0][0]X[2],
		  [0][0]Nrm[0], [0][0]Nrm[1], [0][0]Nrm[2],  [0][0]Iclr );
         img = 0;					/* Background	*/
         where ( Iclr > 0 )
	   Surfprj(&img, &Vw, X, Nrm, Iclr);	/* Project colors	*/
       }
      Xdraw(img&0377);		/* Extracts color from packed form	*/
      printf("Type p <ret> to proceed > ");
      scanf("%1s", &m);
    }
 }
